#include <p18f2321.h>
#include "PinDefs.h"
#include <delays.h>
#include <GenericTypeDefs.h>

volatile unsigned int BP; //current sample filtered pressure
volatile unsigned int BPraw; //current sample unfiltered pressure
volatile unsigned int BPsys; //Systolic blood pressure in MAP algorithm
volatile unsigned char BPsys_max; //Systolic cutoff
volatile unsigned int BPdia; //Diastolic blood pressure in MAP algorithm
volatile unsigned char BPdia_max; //Diastolic cutoff
volatile unsigned char raw[40]; //holds raw pressure
volatile unsigned char press[40];//holds filter pressure
volatile unsigned char count; //pressure measurement counter
volatile unsigned char BPmap; //MAP blood pressure
volatile unsigned char Tcount; //Timer0 counter for power LED
volatile unsigned int ADCtemp; //Temporary ADC storage
volatile unsigned char overthresh;//1 ascending peak, 0 = descending peak
volatile unsigned int BPmax;//Maximum pressure to inflate cuff to
volatile unsigned char state;  //MAP algorithm state variable.
								//0 -> Need to inflate cuff
								//1 -> Cuff deflating record pressure
								//2 -> Fine MAP estimate
								//3 -> Calculate Systolic and Diastolic pressures


//volatile unsigned int press[30];
    #define del  12

extern void setupTmrs(void);

void peaks(void);
unsigned char adc2press(unsigned int adc_val);//converts adc value to mmhg pressure value
char findSys(void); //finds systolic blood pressure
void findBP(void);
unsigned char setDia(unsigned int adc_val);
unsigned char setSys(unsigned int adc_val);
unsigned int setBPmax(unsigned int adc_val);


extern void dispMap(void);
extern void dispDia(void);
extern void dispSys(void);
extern void dispDia_max(void);
extern void dispSys_max(void);

void main (void)
{
  char i; 

  //Initialize variables
  Tcount = 0;
  BP = 0;
  BPraw = 0;
  BPsys = 0;
  BPdia = 0;
  BPmap = 0;
  count = 1;//Pressure index 
  state = 0;
  overthresh = 0;

  for (i=0;i<40;i++)
  {
	raw[i] = 0;	
	press[i] = 0;
  }
  raw[0] = 200;

  // -------------------------------DIGITAL IO--------------------------------------------
   
  //Set port B to Digital Out
  TRISB = 0b00000000;
 
  //Turn All LEDS on for short time to show device on

  //-------------------------------LIGHT AND LEDS AND SETUP DEVICES------------------------

  power = 0;
  thresh = 0;
  BPhigh = 0;
  BPgood = 0;


  setupTmrs();
  //Make PORTA0 input for A/D converter
  TRISAbits.TRISA0 = 1;
  TRISAbits.TRISA1 = 1;
  TRISAbits.TRISA2 = 1;
  TRISAbits.TRISA3 = 1;

  //Set the ADC for internal RC clock
  ADCON2bits.ADCS2 = 1;
  ADCON2bits.ADCS1 = 1;
  ADCON2bits.ADCS0 = 1;

  ADCON0 = 0;

  //Set the ADC to use VDD and GND as references and AN0 - AN4 as A/D inputs
  ADCON1 = 0b00001010;

  //Delay 10000*4/32e3 = 1.25s
  Delay10KTCYx (20);
  //Turn off lights to save power

  thresh = 1;
  BPhigh = 1;
  BPgood = 1;

  //left justify ADC data
  ADCON2bits.ADFM = 1;

 //--------------------------SAMPLE DIASTOLIC AND SYSTOLIC VALUES--------------------------

  //Potentiometers on 
  //pots = 0;
  //threshpot = 0;

   //Set the ADC to sample AN2 Diastolic value
  ADCON0bits.CHS3 = 0;
  ADCON0bits.CHS2 = 0;
  ADCON0bits.CHS1 = 1;
  ADCON0bits.CHS0 = 0;

  //Start conversion
  ADCON0bits.ADON = 1;
  ADCON0bits.GO = 1;
  Delay100TCYx (5);
  while (ADCON0bits.GO)
     ;
  ADCtemp = ADRES;
  
  BPdia_max = setDia(ADCtemp);  
  
   //Set the ADC to sample AN3 Systolic value
  ADCON0bits.CHS3 = 0;
  ADCON0bits.CHS2 = 0;
  ADCON0bits.CHS1 = 1;
  ADCON0bits.CHS0 = 1;

  //Start conversion
  ADCON0bits.ADON = 1;
  ADCON0bits.GO = 1;
  Delay100TCYx (5);
  while (ADCON0bits.GO)
     ;
  ADCtemp = ADRES;
  BPsys_max = setSys(ADCtemp);

  
  BPmax = 160;

 //potentiometers off
  //pots = 1;
  //threshpot = 1;
  //-------------------------------SWITCH OFF LEDS AND START ALGORITHM--------------------------

  thresh = 1;
  BPgood = 1;
  BPhigh = 1;
  //Start conversion
  ADCON0bits.ADON = 1;


  while (1) 
 {  
    //Set ADC to channel AN0
  	ADCON0bits.CHS3 = 0;
  	ADCON0bits.CHS2 = 0;
  	ADCON0bits.CHS1 = 0;
  	ADCON0bits.CHS0 = 0;
    Delay100TCYx (2);
	ADCON0bits.ADON = 1;
	ADCON0bits.GO = 1;
    Delay100TCYx (2);
    while (ADCON0bits.GO)
      ;
    BP = ADRES;
    	
    //Set ADC to channel AN1
  	ADCON0bits.CHS3 = 0;
  	ADCON0bits.CHS2 = 0;
  	ADCON0bits.CHS1 = 0;
  	ADCON0bits.CHS0 = 1;
    Delay100TCYx (2);
	ADCON0bits.ADON = 1;
    ADCON0bits.GO = 1;
    Delay100TCYx (2);
    while (ADCON0bits.GO)
      ;
    BPraw = ADRES;

	if (state == 0)
 	{
		if(adc2press(BPraw) > BPmax+15) //start algorithm
		{
			thresh = 0;
			state = 1;
            BPhigh = 1;
            BPgood = 1;
		}
		else
			thresh = 1;
	}
	if (state == 1)
	{
		if(adc2press(BPraw) < BPmax)
			state = 2;	
	}
	if (state == 2)
    {
		peaks();
		if (adc2press(BPraw) < 55)
			state = 3;

	}
    if (state == 3)
		findBP();	
 }
}

void peaks(void)
{
	#define offset 518
    #define zero 509
   	//Find Peak
   	if (BP>offset) //BP pulse bigger than threshold	
   	{	
        BPhigh = 0;
        BPgood = 0;
		overthresh = 1;
		if (BP-zero>press[count])
		{
			press[count] = BP-zero;		
			raw[count] = adc2press(BPraw);		
		}
	}
	else
	{
        BPhigh = 1;
        BPgood = 1;
		if (overthresh == 1)
		{
			overthresh = 0;
			count++;
		}
	}
		
    if (count == 40)
	{
		state = 3;//algorithm has finished
	}   
}


void findBP(void)
{
   char i = 0;
   char j = 0;
   #define Rsys	0.50
   #define Rdia 0.85
 
   BPmap = 0;

   for (i = 1;i<40;i++)
	{
		if (press[i]>BPmap)
		{
			BPmap = press[i];
			j = i;		
		}

	}
    
    for (i=j;i>0;i--)
	{
		if ((press[i-1] <= Rsys*BPmap) & (press[i] > Rsys*BPmap))
		{
			//BPsys = raw[i-1];
			BPsys = (unsigned char)(raw[i-1]-((Rsys*BPmap-press[i-1])/(press[i]-press[i-1]))*(raw[i-1]-raw[i]));
		}
	
	}
    for (i=j+1;i<40;i++)
	{
		if ((press[i-1] > Rdia*BPmap)&(press[i] <=Rdia*BPmap))
		{
			//BPdia = raw[i-1];
			BPdia = (unsigned char)(raw[i]+(Rdia*BPmap-press[i])/(press[i-1]-press[i])*(raw[i-1]-raw[i]));
		}
	
	}

	if ((BPsys > BPsys_max)||( BPdia > BPdia_max) )
	{
		BPhigh = 0;
		BPgood = 1;
	}
	else
	{
		BPhigh = 1;
		BPgood = 0;
	}
	
    thresh = 1;
    Delay10KTCYx (2*del);	
    BP = 0;     
    BPraw = 0;
    BPsys = 0;
    BPdia = 0;
    BPmap = 0;
    count = 1;//Pressure index 
    state = 0;
    overthresh = 0;
    for (i=0;i<40;i++)
    {
  	 	raw[i] = 0;	
		press[i] = 0;
  	}
  	raw[0] = 200;	
}


//convert adc value to blood pressure value so fits in char
unsigned char adc2press(unsigned int adc_val)
{
    return (unsigned char)(0.42*adc_val-20);
}

//convert the Systolic potentiometer value to systolic limit
unsigned char setSys(unsigned int adc_val)
{
  //return (unsigned char)(adc_val/12.78+100);
    if (adc_val < 5)
		adc_val = 5;
	return (unsigned char)((adc_val-5)/11.45+100);
	//return (unsigned char)(adc_val/4);
}

//convert the Diastolic potentiometer value to the diastolic limit
unsigned char setDia(unsigned int adc_val)
{
	//return (unsigned char)(adc_val/4);
    if (adc_val < 44)
		adc_val = 44;
	return (unsigned char)((adc_val-44)/18.48+60);
  //return (unsigned char)(adc_val/20.46+60);
}

//Converts AN4 to the BP max value
unsigned int setBPmax(unsigned int adc_val)
{
	//return (unsigned char)(adc_val/4);
    if (adc_val < 2)
		adc_val = 2;
	return (unsigned int)((adc_val-2)/9.95+150);
  //return (unsigned char)(adc_val/20.46+60);
}